/*
 Copyright (c) 2020 gematik GmbH

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
package de.gematik.ti.sgd.domain.json;

import com.fasterxml.jackson.annotation.JsonProperty;
import de.gematik.ti.sgd.domain.sgdCommunication.EncryptedMessage;
import lombok.Getter;
import lombok.Setter;

import java.util.Map;

@Getter
@Setter
public class EncryptedMessageContainer {
    private SgdResponseStatus status;
    private EncryptedMessage encryptedMessage;
    private String debug_encryptedMessageCleartext;
    private String debug_clientEphemeralKey_Public;
    private String debug_senderKey_Public;
    private Map<String, String> debug_data;

    public EncryptedMessageContainer() {
    }

    public EncryptedMessageContainer(EncryptedMessage encryptedMessage, SgdResponseStatus status) {
        this.status = status;
        this.encryptedMessage = encryptedMessage;
    }

    public EncryptedMessageContainer(EncryptedMessage encryptedMessage) {
        this.status = SgdResponseStatus.OK;
        this.encryptedMessage = encryptedMessage;
    }

    public static final class EncryptedMessageContainerBuilder {
        private SgdResponseStatus status;
        private EncryptedMessage encryptedMessage;
        private String debug_encryptedMessageCleartext;
        private String debug_clientEphemeralKey_Public;
        private String debug_senderKey_Public;
        private Map<String, String> debug_data;

        private EncryptedMessageContainerBuilder() {
        }

        public static EncryptedMessageContainerBuilder anEncryptedMessageContainer() {
            return new EncryptedMessageContainerBuilder();
        }

        public EncryptedMessageContainerBuilder withStatus(SgdResponseStatus status) {
            this.status = status;
            return this;
        }

        public EncryptedMessageContainerBuilder withEncryptedMessage(EncryptedMessage encryptedMessage) {
            this.encryptedMessage = encryptedMessage;
            return this;
        }

        public EncryptedMessageContainerBuilder withDebug_encryptedMessageCleartext(String debug_encryptedMessageCleartext) {
            this.debug_encryptedMessageCleartext = debug_encryptedMessageCleartext;
            return this;
        }

        public EncryptedMessageContainerBuilder withDebug_clientEphemeralKey_Public(String debug_clientEphemeralKey_Public) {
            this.debug_clientEphemeralKey_Public = debug_clientEphemeralKey_Public;
            return this;
        }

        public EncryptedMessageContainerBuilder withDebug_senderKey_Public(String debug_senderKey_Public) {
            this.debug_senderKey_Public = debug_senderKey_Public;
            return this;
        }

        public EncryptedMessageContainerBuilder withDebug_data(Map<String, String> debug_data) {
            this.debug_data = debug_data;
            return this;
        }

        public EncryptedMessageContainer build() {
            EncryptedMessageContainer encryptedMessageContainer = new EncryptedMessageContainer(encryptedMessage, status);
            encryptedMessageContainer.setDebug_encryptedMessageCleartext(debug_encryptedMessageCleartext);
            encryptedMessageContainer.setDebug_data(debug_data);
            encryptedMessageContainer.debug_clientEphemeralKey_Public = this.debug_clientEphemeralKey_Public;
            encryptedMessageContainer.debug_senderKey_Public = this.debug_senderKey_Public;
            return encryptedMessageContainer;
        }
    }
}
