/*
 Copyright (c) 2020 gematik GmbH

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
package de.gematik.ti.sgd.domain.json;

import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import de.gematik.ti.sgd.crypto.SgdCryptoUtilities;
import de.gematik.ti.sgd.domain.sgdCommunication.EncryptedMessage;
import de.gematik.ti.sgd.domain.sgdCommunication.PublicKeyEcies;
import de.gematik.ti.sgd.exceptions.server.SgdServerException;
import lombok.Builder;
import lombok.Data;

import java.security.cert.X509Certificate;

@Data
@Builder
public class SgdRequest {
    @JsonProperty(value = "Command", required = true)
    private SgdCommand command;
    @JsonProperty(value = "PublicKeyECIES", required = true)
    private PublicKeyEcies publicKeyEcies;
    @JsonProperty(value = "Signature", required = true)
    private String signature;
    @JsonProperty(value = "Certificate", required = true)
    private String certificate;
    @JsonProperty(value = "EncryptedMessage", required = true)
    private EncryptedMessage encryptedMessage;
    @JsonProperty(value = "OCSPResponse")
    private String ocspResponse;

    public SgdRequest() {
    }

    public SgdRequest(SgdCommand command, PublicKeyEcies publicKeyEcies, String signature, String certificate, EncryptedMessage encryptedMessage, String ocspResponse) {
        this.command = command;
        this.publicKeyEcies = publicKeyEcies;
        this.signature = signature;
        this.certificate = certificate;
        this.encryptedMessage = encryptedMessage;
        this.ocspResponse = ocspResponse;
    }

    @JsonIgnore
    public X509Certificate getCertificateAsX509() {
        try {
            return SgdCryptoUtilities.getInstance().parseX509CertificateFromB64String(certificate);
        } catch (RuntimeException e) {
            throw new SgdServerException(SgdResponseStatus.CERTIFICATE_NOT_VALID, e);
        }
    }
}
