/*
 Copyright (c) 2020 gematik GmbH

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
package de.gematik.ti.sgd.crypto;

import de.gematik.ti.sgd.SgdTestCryptoUtilities;
import de.gematik.ti.sgd.domain.sgdCommunication.PublicKeyEcies;
import org.junit.Ignore;
import org.junit.Test;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.PublicKey;
import java.security.spec.PKCS8EncodedKeySpec;
import java.security.spec.X509EncodedKeySpec;

import static org.assertj.core.api.AssertionsForClassTypes.assertThat;

public class PointCoordinatesConversionTest {
    private static final Logger logger = LoggerFactory.getLogger(PointCoordinatesConversionTest.class);

    {
        SgdCryptoUtilities.setInstance(new SgdTestCryptoUtilities());
    }

    @Test
    @Ignore
    public void bruteForce() {
        for (int i = 0; i < 10_000; i++) {
            if (i % 100 == 0) {
                logger.info("Durchgang " + i);
            }
            final KeyPair keyPair = SgdCryptoUtilities.getInstance().generateEccKeyPair();
            final String coordinates = SgdCryptoUtilities.getInstance().extractPublicPointCoordinatesFromPublicKey(keyPair.getPublic());
            if (coordinates.contains("0x00")) {
                System.out.println(coordinates);
                System.out.println();
                System.out.println();
                System.out.println(
                        SgdCryptoUtilities.getInstance().convertBytesToB64String(keyPair.getPublic().getEncoded()));
            }
        }
    }

    @Test
    //gemSpec_Sgd, A_18250
    public void extractCoordinatesWithLeadingZeros_EncodingShouldNotContainLeadingZeros() throws Exception {
        String b64EncodedKey = "MFowFAYHKoZIzj0CAQYJKyQDAwIIAQEHA0IABJGPxNVMFpbjTLtT9bomdJg5IuVpucYNMbdnB+EQ8l/hAKvb1iXHnJ8fJQW0KqhsisGZ4GmweU4oTbT4CYbm4X8=";
        final byte[] keyBytes = SgdCryptoUtilities.getInstance().convertB64StringToBytes(b64EncodedKey);
        PublicKey publicKey = loadPublicKeyFromBytes(keyBytes);
        assertThat(SgdCryptoUtilities.getInstance().extractPublicPointCoordinatesFromPublicKey(publicKey))
                .doesNotContain("0x0");
    }

    @Test
    //gemSpec_Sgd, A_18250
    public void reconstructCoordinatesWithLeadingZeros() throws Exception {
        String b64EncodedKey = "MFowFAYHKoZIzj0CAQYJKyQDAwIIAQEHA0IABJGPxNVMFpbjTLtT9bomdJg5IuVpucYNMbdnB+EQ8l/hAKvb1iXHnJ8fJQW0KqhsisGZ4GmweU4oTbT4CYbm4X8=";
        final byte[] keyBytes = SgdCryptoUtilities.getInstance().convertB64StringToBytes(b64EncodedKey);
        PublicKey publicKey = loadPublicKeyFromBytes(keyBytes);

        final String pointCoordinates = "brainpoolP256r1 " + SgdCryptoUtilities.getInstance().extractPublicPointCoordinatesFromPublicKey(publicKey);
        PublicKeyEcies publicKeyEcies = new PublicKeyEcies(pointCoordinates);
        assertThat(publicKeyEcies.getPublicKey())
                .isEqualTo(publicKey);
    }

    private PublicKey loadPublicKeyFromBytes(byte[] keyBytes) throws Exception {
        X509EncodedKeySpec keySpec = new X509EncodedKeySpec(keyBytes);
        KeyFactory kf = KeyFactory.getInstance("ECDSA");
        return kf.generatePublic(keySpec);
    }
}