/*
 * Copyright (c) 2019 gematik - Gesellschaft für Telematikanwendungen der Gesundheitskarte mbH
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package de.gematik.ti.sgd.server;

import de.gematik.ti.sgd.components.SchluesselGenerierungsDienst;
import de.gematik.ti.sgd.components.SgdClient;
import de.gematik.ti.sgd.components.Sgd1ServerWrapper;
import de.gematik.ti.sgd.components.Sgd2ServerWrapper;
import de.gematik.ti.sgd.domain.configuration.SgdClientProperties;
import de.gematik.ti.sgd.domain.configuration.SgdInstanceProperties;
import de.gematik.ti.sgd.domain.json.SgdRequest;
import de.gematik.ti.sgd.domain.json.SgdResponse;
import org.junit.Before;
import org.junit.runner.RunWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.web.client.TestRestTemplate;
import org.springframework.test.context.ActiveProfiles;
import org.springframework.test.context.TestPropertySource;
import org.springframework.test.context.junit4.SpringRunner;
import org.springframework.test.util.ReflectionTestUtils;
import org.springframework.web.client.RestTemplate;
import org.springframework.web.util.DefaultUriBuilderFactory;

import java.io.File;
import java.security.cert.X509Certificate;
import java.util.Arrays;
import java.util.Collections;

@RunWith(SpringRunner.class)
@SpringBootTest(webEnvironment = SpringBootTest.WebEnvironment.RANDOM_PORT)
@TestPropertySource(properties = "sgd.isSgd1Instance = true")
@ActiveProfiles("test")
public abstract class TestWithConfiguredSgdClient {
    public SgdClient sgdClient;
    public Sgd1ServerWrapper wrapper1;
    public Sgd2ServerWrapper wrapper2;
    @Autowired
    public TestRestTemplate testRestTemplate;
    public X509Certificate clientCertificate;
    public SchluesselGenerierungsDienst sgd2;

    @Before
    public void setUpClient() {
        SgdInstanceProperties properties = new SgdInstanceProperties();
        properties.setCertificateP12Password("00");
        properties.setCertificateP12("src/main/resources/c.sgd2.hsm-aut-ecc_in_TSL.p12");
        properties.setCurrentDerivationKeyName("yamlKeySgd2");
        properties.setCurrentDerivationKeyValue("312FC332BBB428BB25CF9A5EF727324CB9168E03E76FA3E92B28CB60F1D85F6229DDC260F42D04D1F345636628B72DFDD8C27C505CE158F41921B4583BA0B2D6");
        properties.setPerformServerSidedHCheck(true);

        sgd2 = new SchluesselGenerierungsDienst(properties, Arrays.asList("src/main/resources/GEM.EGK-CA08-TEST-ONLY.crt"), Collections.emptyMap());
        wrapper1 = new LocalTestServer1Wrapper();
        wrapper2 = new LocalTestServer2Wrapper();
        sgdClient = new SgdClient(SgdClientProperties.builder()
                .clientCertificateFile("src/test/resources/egk_aut_rsa.crt")
                .clientKeyPairFile("src/test/resources/egk_aut_rsa.p12")
                .possibleSignerCertificateFiles(Arrays.asList(
                        "src/main/resources/GEM.KOMP-CA17-TEST-ONLY.crt",
                        "src/main/resources/GEM.KOMP-CA10-TEST-ONLY.crt"))
                .build(), wrapper1, wrapper2);

        clientCertificate = (X509Certificate) ReflectionTestUtils.getField(sgdClient, "clientCertificate");
    }

    private class LocalTestServer1Wrapper implements Sgd1ServerWrapper {
        @Override
        public SgdResponse sgd1Endpoint(SgdRequest request) {
            return testRestTemplate.postForObject("/sgd1", request, SgdResponse.class);
        }

    }

    private class LocalTestServer2Wrapper implements Sgd2ServerWrapper {
        @Override
        public SgdResponse sgd2Endpoint(SgdRequest request) {
            return sgd2.receive(request);
        }
    }
}
